<?php
/*
Plugin Name: Gerador de Webstory
Plugin URI: https://seusite.com
Description: Adiciona a opção "Gerar Webstory" na lista de posts.
Version: 1.0
Author: Seu Nome
Author URI: https://seusite.com
*/

// Adiciona a opção "Gerar Webstory" no menu de ações dos posts se o post não for uma webstory
function webstory_add_action_link($actions, $post)
{
    if ($post->post_type != 'web-story') {
        return $actions;
    }

    $url = wp_nonce_url(admin_url('admin-post.php?action=generate_webstory&post_id=' . $post->ID), 'generate_webstory');
    $actions['generate_webstory'] = '<a href="' . $url . '">Gerar Webstory</a>';
    return $actions;
}
add_filter('post_row_actions', 'webstory_add_action_link', 10, 2);

// Gerar webstory
function generate_webstory($post){
    $post_id = $post->ID;
    $post = get_post($post_id);

    // post featured image
    $post_thumbnail_id = get_post_thumbnail_id($post_id);
    if ($post_thumbnail_id) {
        $post_thumbnail_url = wp_get_attachment_url($post_thumbnail_id);
    } else {
        $post_thumbnail_url = '';
    }

    // post url
    $post_url = get_permalink($post_id);

    // get full post content in HTML
    $post_content = apply_filters('the_content', $post->post_content);

    // send post data to webstory generator as json
    $data = array(
        'post_title' => $post->post_title,
        'post_content' => $post_content,
        'post_excerpt' => $post->post_excerpt,
        'post_thumbnail_url' => $post_thumbnail_url,
        'post_author' => $post->post_author,
        'post_url' => $post_url
    );

    $data = json_encode($data);

    $response = wp_remote_post('https://webstorygenerator.yan.dev.br/pro', array(
        'headers' => array('Content-Type' => 'application/json'),
        'timeout'     => 60,
        'body' => $data
    )
    );

    // check if response status is 200
    if (wp_remote_retrieve_response_code($response) != 200) {
        return false;
    }

    if (is_wp_error($response)) {
        return false;
    }

    $body = wp_remote_retrieve_body($response);
    // remove first and last double quotes
    $body = substr($body, 1, -1);

    $new_post = array(
        'post_title' => $post->post_title,
        'post_content_filtered' => $body,
        'post_excerpt' => $post->post_excerpt,
        'post_status' => 'draft',
        'post_type' => 'web-story'
    );

    $new_post_id = wp_insert_post($new_post);

    if ($new_post_id) {
        // set post featured image
        if ($post_thumbnail_id) {
            set_post_thumbnail($new_post_id, $post_thumbnail_id);
        }
        // set post category
        $post_categories = wp_get_post_categories($post_id);
        wp_set_post_categories($new_post_id, $post_categories);

        // redirect to post edit page
        $post_edit_page = admin_url('post.php?action=edit&post=' . $new_post_id);
        return $post_edit_page;
    }

    return false;

}

// Processa a requisição de geração de webstory
function webstory_generate_webstory()
{
    if (!isset($_GET['post_id']) || !wp_verify_nonce($_REQUEST['_wpnonce'], 'generate_webstory')) {
        wp_die('Acesso inválido.');
    }

    $post = get_post($_GET['post_id']);

    $post_edit_page = generate_webstory($post);

    if ($post_edit_page) {
        wp_redirect($post_edit_page . '&sucess=1');
    } else {
        wp_redirect(admin_url('edit.php?post_type=web-story&webstory_status=error'));
    }
}

add_action('admin_post_generate_webstory', 'webstory_generate_webstory');

// Mostra a mensagem de erro se não foi possível gerar a webstory
function webstory_show_message()
{
    if (!isset($_GET['webstory_status'])) {
        return;
    }

    $status = $_GET['webstory_status'];

    if ($status == 'error') {
        $class = 'notice notice-error';
        $message = 'Não foi possível gerar o webstory.';
        printf('<div class="%1$s"><p>%2$s</p></div>', esc_attr($class), $message);
    }
}
add_action('admin_notices', 'webstory_show_message');

// Mosta a mensagem de sucesso se a webstory foi gerada com sucesso
function webstory_show_success_message()
{
    if (!isset($_GET['sucess'])) {
        return;
    }

    $class = 'notice notice-success';
    $message = 'Webstory gerada com sucesso.';
    printf('<div class="%1$s"><p>%2$s</p></div>', esc_attr($class), $message);
}

add_action('admin_notices', 'webstory_show_success_message');

// Adiciona a opção "Gerar Webstory" no menu de ações dos posts
function webstory_add_action_link_webstory($actions, $post)
{
    $url = wp_nonce_url(admin_url('admin-post.php?action=generate_webstory&post_id=' . $post->ID), 'generate_webstory');
    $actions['generate_webstory'] = '<a href="' . $url . '">Gerar Webstory</a>';
    return $actions;
}

add_filter('post_row_actions', 'webstory_add_action_link_webstory', 10, 2);
add_filter('page_row_actions', 'webstory_add_action_link_webstory', 10, 2);

// Verifica se o plugin web-stories está ativo e instalado
function webstory_check_plugin()
{
    if (!is_plugin_active('web-stories/web-stories.php')) {
        add_action('admin_notices', 'webstory_plugin_notice');
    }
}

add_action('admin_init', 'webstory_check_plugin');

// Mostra mensagem de erro se o plugin web-stories não estiver ativo
function webstory_plugin_notice()
{
    $class = 'notice notice-error';
    $message = 'O plugin <strong>Web Stories</strong> não está ativo. Por favor, ative-o para poder gerar webstories.';

    printf('<div class="%1$s"><p>%2$s</p></div>', esc_attr($class), $message);
}

// call webstory_generate_webstory function when post is published
function webstory_generate_webstory_on_publish($new_status, $old_status, $post)
{
    if ($new_status == 'publish' && $old_status != 'publish' && $post->post_type != 'web-story') {
        generate_webstory($post);
    }
}

add_action('transition_post_status', 'webstory_generate_webstory_on_publish', 10, 3);